from datetime import timedelta
from unittest.mock import patch

import pandas as pd
import phonenumbers
import pytest
from rest_framework import status
from wbcore.contrib.directory.factories import TelephoneContactFactory
from wbintegrator_office365.factories import SubscriptionFactory, TenantUserFactory
from wbintegrator_office365.importer import parse
from wbintegrator_office365.models import CallEvent, Subscription
from wbintegrator_office365.models.event import fetch_tenantusers
from wbintegrator_office365.models.subscription import (
    subscribe,
    unsubscribe,
    verification_subscriptions,
)


# This method will be used by the mock to replace requests.get
class MockResponse:
    def __init__(self, json_data, status_code):
        self.json_data = json_data
        self.status_code = status_code

    def json(self):
        return self.json_data


@pytest.mark.django_db
class TestTaskSubscription:
    @patch("wbintegrator_office365.importer.MicrosoftGraphAPI._get_access_token")
    @patch("wbintegrator_office365.importer.api.requests.post")
    def test_subscribe(self, mock_post, mock_access_token):
        obj = SubscriptionFactory(type_resource=Subscription.TypeResource.CALLRECORD)
        data = {
            "@odata.context": "https://graph.microsoft.com/v1.0/$metadata#subscriptions/$entity",
            "value": [
                {
                    "id": "7f105c7d-2dc5-4530-97cd-4e7ae6534c07",
                    "resource": "me/mailFolders('Inbox')/messages",
                    "applicationId": "24d3b144-21ae-4080-943f-7067b395b913",
                    "changeType": "created",
                    "clientState": "secretClientValue",
                    "notificationUrl": "https://webhook.azurewebsites.net/api/send/myNotifyClient",
                    "expirationDateTime": "2016-11-20T18:23:45.9356913Z",
                    "creatorId": "8ee44408-0679-472c-bc2a-692812af3437",
                    "latestSupportedTlsVersion": "v1_2",
                }
            ],
        }
        data = parse(pd.json_normalize(data.get("value")))[0]
        mock_access_token.return_value.status_code = status.HTTP_200_OK
        mock_access_token.return_value = "FAKE_TOKEN"

        mock_post.return_value = MockResponse(data, 201)
        assert obj.subscription_id != data.get("id")
        subscribe(obj.id, "/communications/callRecords")
        assert mock_post.call_count == 1
        obj = Subscription.objects.get(id=obj.id)
        assert obj.subscription_id == data.get("id")

    @patch("wbintegrator_office365.importer.MicrosoftGraphAPI.users")
    @patch("wbintegrator_office365.models.subscription.chain")
    def test_post_save_subscription(self, mock_chain, mock_users):
        with patch("wbintegrator_office365.models.event.transaction.on_commit", new=lambda fn: fn()):
            mock_chain.return_value.status_code = 200
            mock_users.return_value.status_code = 200

            tenant = TenantUserFactory()
            mock_users.return_value = [{"id": tenant.tenant_id}]

            assert mock_chain.call_count == 0
            SubscriptionFactory(
                is_enable=True, type_resource=Subscription.TypeResource.CALLRECORD, expiration_date=None
            )
            assert mock_chain.call_count == 1
            SubscriptionFactory(
                subscription_id=None,
                is_enable=True,
                tenant_user=tenant,
                type_resource=Subscription.TypeResource.CALENDAR,
            )
            assert mock_chain.call_count == 1

    @pytest.mark.parametrize("type_resource", ["CALLRECORD", "CALENDAR"])
    @patch("wbintegrator_office365.models.subscription.chain")
    @patch("wbintegrator_office365.importer.MicrosoftGraphAPI.users")
    @patch("requests.patch")
    def test_resubscribe(self, mock_patch, mock_users, mock_chain, type_resource):
        if type_resource == Subscription.TypeResource.CALLRECORD:
            obj = SubscriptionFactory(is_enable=True, type_resource=Subscription.TypeResource.CALLRECORD)
        else:
            mock_users.return_value.status_code = 200
            tenant = TenantUserFactory()
            mock_users.return_value = [{"id": tenant.tenant_id}]
            obj = SubscriptionFactory(
                is_enable=True, tenant_user=tenant, type_resource=Subscription.TypeResource.CALENDAR
            )
        data = {"expiration": obj.expiration_date + timedelta(days=2), "notification_url": "fake_url"}
        mock_patch.return_value.status_code = 200
        mock_patch.return_value.json.return_value = data

        assert mock_patch.call_count == 0
        obj.resubscribe()
        assert mock_patch.call_count == 1

        obj = Subscription.objects.get(id=obj.id)
        assert obj.expiration_date == data.get("expiration")

    @patch("wbintegrator_office365.models.subscription.chain")
    @patch("requests.get")
    def test_verification_subscriptions(self, mock_get, mock_chain):
        with patch("wbintegrator_office365.models.event.transaction.on_commit", new=lambda fn: fn()):
            data = {"value": [{"id": "fake_id"}]}
            mock_get.return_value.status_code = 200
            mock_get.return_value.json.return_value = data

            obj = SubscriptionFactory(
                expiration_date=None, type_resource="CALLRECORD", is_enable=True, subscription_id="fake_id"
            )
            obj2 = SubscriptionFactory(
                subscription_id=None, type_resource="CALENDAR", is_enable=True, tenant_user=TenantUserFactory()
            )
            obj2.subscription_id = "other_fake_id"
            obj2.save()

            assert mock_chain.call_count == 1
            assert mock_get.call_count == 0
            verification_subscriptions(obj.id)
            assert mock_get.call_count == 1
            verification_subscriptions(obj2.id)
            assert mock_get.call_count == 2

            assert Subscription.objects.get(id=obj.id).is_enable is True
            assert Subscription.objects.get(id=obj2.id).is_enable is False

    @patch("requests.delete")
    def test_unsubscribe(self, mock_delete):
        mock_delete.return_value.status_code = 204
        obj = SubscriptionFactory(is_enable=True)
        assert mock_delete.call_count == 0
        unsubscribe(obj.id)
        assert mock_delete.call_count == 1


data = {
    "@odata.context": "https://graph.microsoft.com/v1.0/$metadata#communications/callRecords/$entity",
    "version": 1,
    "type": "peerToPeer",
    "modalities": ["audio"],
    "lastModifiedDateTime": "2020-02-25T19:00:24.582757Z",
    "startDateTime": "2020-02-25T18:52:21.2169889Z",
    "endDateTime": "2020-02-25T18:52:46.7640013Z",
    "id": "e523d2ed-2966-4b6b-925b-754a88034cc5",
    "organizer": {
        "user": {
            "id": "821809f5-0000-0000-0000-3b5136c0e777",
            "displayName": "Abbie Wilkins",
            "tenantId": "dc368399-474c-4d40-900c-6265431fd81f",
        }
    },
    "participants": [
        {
            "user": {
                "id": "821809f5-0000-0000-0000-3b5136c0e777",
                "displayName": "Abbie Wilkins",
                "tenantId": "dc368399-474c-4d40-900c-6265431fd81f",
            }
        },
        {
            "user": {
                "id": "f69e2c00-0000-0000-0000-185e5f5f5d8a",
                "displayName": "Owen Franklin",
                "tenantId": "dc368399-474c-4d40-900c-6265431fd81f",
            }
        },
    ],
}

data_with_guest_user = {
    "@odata.context": "https://graph.microsoft.com/v1.0/$metadata#communications/callRecords/$entity",
    "version": 1,
    "type": "peerToPeer",
    "modalities": ["audio"],
    "lastModifiedDateTime": "2020-02-25T19:00:24.582757Z",
    "startDateTime": "2020-02-25T18:52:21.2169889Z",
    "endDateTime": "2020-02-25T18:52:46.7640013Z",
    "id": "e523d2ed-2966-4b6b-925b-754a88034cc5",
    "organizer": {
        "user": {
            "id": "821809f5-0000-0000-0000-3b5136c0e777",
            "displayName": "Abbie Wilkins",
            "tenantId": "dc368399-474c-4d40-900c-6265431fd81f",
        }
    },
    "participants": [
        {
            "user": None,
            "acsUser": None,
            "spoolUser": None,
            "phone": None,
            "guest": {"id": "c6f4f18f1e99401a9193fbf954f6e903", "displayName": "Guest user", "tenantId": None},
            "encrypted": None,
            "onPremises": None,
            "acsApplicationInstance": None,
            "spoolApplicationInstance": None,
            "applicationInstance": None,
            "application": None,
            "device": None,
        },
        {
            "acsUser": None,
            "spoolUser": None,
            "phone": None,
            "guest": None,
            "encrypted": None,
            "onPremises": None,
            "acsApplicationInstance": None,
            "spoolApplicationInstance": None,
            "applicationInstance": None,
            "application": None,
            "device": None,
            "user": {
                "id": "303eecac-5bf1-44df-96c2-2aae187d46d1",
                "displayName": "External user",
                "tenantId": "9692a3d3-2a08-4ec8-a0bd-1db355eb4230",
            },
        },
    ],
}

data_with_guest_organiser = {
    "@odata.context": "https://graph.microsoft.com/v1.0/$metadata#communications/callRecords/$entity",
    "version": 1,
    "type": "peerToPeer",
    "modalities": ["audio"],
    "lastModifiedDateTime": "2020-02-25T19:00:24.582757Z",
    "startDateTime": "2020-02-25T18:52:21.2169889Z",
    "endDateTime": "2020-02-25T18:52:46.7640013Z",
    "id": "e523d2ed-2966-4b6b-925b-754a88034cc5",
    "organizer": {
        "user": None,
        "guest": {
            "id": "821809f5-0000-0000-0000-3b5136c0e777",
            "displayName": "Abbie Wilkins",
            "tenantId": "dc368399-474c-4d40-900c-6265431fd81f",
        },
    },
    "participants": [],
}

data_with_phone_organiser = {
    "@odata.context": "https://graph.microsoft.com/v1.0/$metadata#communications/callRecords/$entity",
    "version": 1,
    "type": "peerToPeer",
    "modalities": ["audio"],
    "lastModifiedDateTime": "2020-02-25T19:00:24.582757Z",
    "startDateTime": "2020-02-25T18:52:21.2169889Z",
    "endDateTime": "2020-02-25T18:52:46.7640013Z",
    "id": "e523d2ed-2966-4b6b-925b-754a88034cc5",
    "organizer": {
        "user": None,
        "guest": None,
        "phone": {"id": "+41223178146", "displayName": "None", "tenantId": "None"},
    },
    "participants": [],
}


@pytest.mark.django_db
class TestTasksCall:
    @patch("wbintegrator_office365.importer.MicrosoftGraphAPI.users")
    def test_fetch_tenantusers(self, mock_users):
        data = [
            {
                "id": "87d349ed-44d7-43e1-9a83-5f2406dee5bd",
                "display_name": "contoso1",
                "email": "contoso1_gmail.com#EXT#@microsoft.onmicrosoft.com",
            }
        ]
        mock_users.return_value.status_code = 200
        mock_users.return_value = data
        datum, count_added = fetch_tenantusers()
        assert mock_users.call_count == 1
        assert datum == data
        assert count_added == 1

    @patch("requests.get")
    @pytest.mark.parametrize(
        "data", [data, data_with_guest_user, data_with_guest_organiser, data_with_phone_organiser]
    )
    def test_fetch_call(self, mock_get, data, event_factory):
        event = event_factory(uuid_event="87d349ed-44d7-43e1-9a83-5f2406dee5bd")
        mock_get.return_value.status_code = 200
        mock_get.return_value.json.return_value = parse(pd.json_normalize(data))

        assert CallEvent.objects.count() == 0
        event.fetch_call()
        assert mock_get.call_count == 1
        assert CallEvent.objects.count() == 1

    @patch("requests.get")
    @pytest.mark.parametrize("data", [data_with_phone_organiser])
    def test_fetch_call_phone(self, mock_get, data, event_factory):
        event = event_factory(uuid_event="87d349ed-44d7-43e1-9a83-5f2406dee5bd")
        mock_get.return_value.status_code = 200
        mock_get.return_value.json.return_value = parse(pd.json_normalize(data))

        my_number = "+41 22 317 81 46"
        parser_number = phonenumbers.parse(my_number, "CH")
        phone_numbers = phonenumbers.format_number(parser_number, phonenumbers.PhoneNumberFormat.E164)
        TelephoneContactFactory(number=phone_numbers)

        assert CallEvent.objects.count() == 0
        event.fetch_call()
        assert mock_get.call_count == 1
        assert CallEvent.objects.count() == 1



from os import environ

from configurations import values


class Office365:
    WBINTEGRATOR_OFFICE365_TENANT_ID = values.Value("", environ_prefix=None)
    WBINTEGRATOR_OFFICE365_CLIENT_ID = values.Value("", environ_prefix=None)
    WBINTEGRATOR_OFFICE365_CLIENT_SECRET = values.Value("", environ_prefix=None)
    WBINTEGRATOR_OFFICE365_REDIRECT_URI = values.Value("", environ_prefix=None)
    WBINTEGRATOR_OFFICE365_TOKEN_ENDPOINT = values.Value("", environ_prefix=None)
    WBINTEGRATOR_OFFICE365_NOTIFICATION_URL = values.Value("", environ_prefix=None)
    WBINTEGRATOR_OFFICE365_GRAPH_API_VERSION = values.Value("", environ_prefix=None)
    WBINTEGRATOR_OFFICE365_AUTHORITY_PREFIX = values.Value("", environ_prefix=None)
    WBINTEGRATOR_OFFICE365_GRAPH_URL_PREFIX = values.Value("", environ_prefix=None)

    @property
    def WBINTEGRATOR_OFFICE365_AUTHORITY(self):
        return self.WBINTEGRATOR_OFFICE365_AUTHORITY_PREFIX + self.WBINTEGRATOR_OFFICE365_TENANT_ID

    @property
    def WBINTEGRATOR_OFFICE365_GRAPH_URL(self):
        return self.WBINTEGRATOR_OFFICE365_GRAPH_URL_PREFIX + self.WBINTEGRATOR_OFFICE365_GRAPH_API_VERSION



from datetime import date, timedelta

from psycopg.types.range import DateRange
from wbcore import filters as wb_filters
from wbhuman_resources.models import EmployeeHumanResource

from wbintegrator_office365.models import (
    CallEvent,
    CallUser,
    Event,
    EventLog,
    Subscription,
    TenantUser,
)


class TenantUserFilter(wb_filters.FilterSet):
    class Meta:
        model = TenantUser
        fields = {
            "profile": ["exact"],
            "display_name": ["exact", "icontains"],
            "mail": ["exact", "icontains"],
            "phone": ["exact", "icontains"],
            "tenant_id": ["exact", "icontains"],
            "tenant_organization_id": ["exact", "icontains"],
            "is_internal_organization": ["exact"],
        }


class SubscriptionFilter(wb_filters.FilterSet):
    class Meta:
        model = Subscription
        fields = {
            "change_type": ["exact", "icontains"],
            "subscription_id": ["exact", "icontains"],
            "expiration_date": ["lte", "gte"],
            "type_resource": ["exact"],
            "tenant_user": ["exact"],
            "is_enable": ["exact"],
            "created": ["lte", "gte"],
        }


def current_year(field, request, view):
    today = date.today()
    return date(today.year, 1, 1)


class EventFilter(wb_filters.FilterSet):
    created__gte = wb_filters.DateFilter(
        label="Created",
        lookup_expr="gte",
        field_name="created",
        method="created_filter",
        initial=current_year,
    )

    def created_filter(self, queryset, name, value):
        if value:
            return queryset.filter(created__gte=value)
        return queryset

    class Meta:
        model = Event
        fields = {
            "id": ["exact"],
            "nb_received": ["exact"],
            "type": ["exact"],
            "change_type": ["exact"],
            "created": ["gte", "lte"],
            "changed": ["gte", "lte"],
            "tenant_user": ["exact"],
            "subscription_id": ["icontains", "exact"],
            "id_event": ["icontains", "exact"],
            "is_handled": ["exact"],
        }


class EventLogFilter(wb_filters.FilterSet):
    class Meta:
        model = EventLog
        fields = {
            "id": ["exact"],
            "order_received": ["exact"],
            "last_event": ["exact"],
            "change_type": ["exact"],
            "id_event": ["icontains", "exact"],
            "created": ["gte", "lte"],
            "changed": ["gte", "lte"],
        }


class CallEventFilter(wb_filters.FilterSet):
    change_type = wb_filters.CharFilter(label="Type", lookup_expr="icontains")
    participants = wb_filters.ModelMultipleChoiceFilter(
        label="Participants",
        queryset=CallUser.objects.all(),
        endpoint=CallUser.get_representation_endpoint(),
        value_key=CallUser.get_representation_value_key(),
        label_key=CallUser.get_representation_label_key(),
        method="filter_participants",
    )
    start__gte = wb_filters.DateFilter(
        label="Start",
        lookup_expr="gte",
        field_name="start",
        method="start_filter",
        initial=current_year,
    )

    def filter_participants(self, queryset, name, value):
        if value:
            return queryset.filter(participants__in=value)
        return queryset

    def start_filter(self, queryset, name, value):
        if value:
            return queryset.filter(start__gte=value)
        return queryset

    class Meta:
        model = CallEvent
        fields = {
            "event": ["exact"],
            "organizer": ["exact"],
            "start": ["gte", "exact", "lte"],
            "end": ["gte", "exact", "lte"],
            "created": ["gte", "exact", "lte"],
            "is_internal_call": ["exact"],
        }


class CallUserFilter(wb_filters.FilterSet):
    name_user = wb_filters.CharFilter(label="Question", lookup_expr="icontains")
    phone = wb_filters.CharFilter(label="Question", lookup_expr="icontains")
    mail = wb_filters.CharFilter(label="Question", lookup_expr="icontains")

    class Meta:
        model = CallUser
        fields = {
            "tenant_user": ["exact"],
            "is_phone": ["exact"],
            "is_guest": ["exact"],
        }


def previous_month(*args, **kwargs):
    return date.today() - timedelta(days=30)


def next_day(*args, **kwargs):
    return date.today() + timedelta(days=1)


class CallEventSummaryGraphFilter(wb_filters.FilterSet):
    date_range = wb_filters.DateTimeRangeFilter(
        method="filter_date_range",
        label="Date Range",
        required=True,
        clearable=False,
        initial=lambda f, v, q: DateRange(previous_month(f, v, q), next_day(f, v, q)),
    )

    employee = wb_filters.ModelChoiceFilter(
        label="Employee",
        queryset=EmployeeHumanResource.objects.all(),
        endpoint=EmployeeHumanResource.get_representation_endpoint(),
        value_key=EmployeeHumanResource.get_representation_value_key(),
        label_key=EmployeeHumanResource.get_representation_label_key(),
        method="filter_employee",
    )

    call_type = wb_filters.ChoiceFilter(
        label="Type of call",
        choices=[("groupCall", "Group Call"), ("peerToPeer", "Peer To Peer")],
        method="filter_call_type",
    )

    call_area = wb_filters.ChoiceFilter(
        label="Call area",
        choices=[("True", "Internal Call"), ("False", "External Call")],
        method="filter_call_area",
    )

    compare_employee = wb_filters.ModelChoiceFilter(
        label="Compare to an employee",
        queryset=EmployeeHumanResource.objects.all(),
        endpoint=EmployeeHumanResource.get_representation_endpoint(),
        value_key=EmployeeHumanResource.get_representation_value_key(),
        label_key=EmployeeHumanResource.get_representation_label_key(),
        method="filter_compare_employee",
    )

    call_duration = wb_filters.NumberFilter(
        label="Include calls of at least how many secondes", initial=30, method="filter_call_duration", required=False
    )

    business_day_without_call = wb_filters.BooleanFilter(
        label="Include business day without call", initial=False, method="filter_business_day_without_call"
    )

    def filter_date_range(self, queryset, name, value):
        if value:
            return queryset.filter(start__gte=value.lower, end__lte=value.upper)
        return queryset

    def filter_employee(self, queryset, name, value):
        if value:
            return queryset.filter(
                participants__tenant_user__profile__computed_str__icontains=value.profile.computed_str
            )
        return queryset

    def filter_call_duration(self, queryset, name, value):
        if value:
            return queryset.filter(duration_seconds__gte=timedelta(seconds=float(value)))
        return queryset

    def filter_call_type(self, queryset, name, value):
        if value:
            return queryset.filter(type=value)
        return queryset

    def filter_call_area(self, queryset, name, value):
        if value is not None:
            return queryset.filter(is_internal_call=value)
        return queryset

    def filter_compare_employee(self, queryset, name, value):
        return queryset

    def filter_business_day_without_call(self, queryset, name, value):
        return queryset

    class Meta:
        model = CallEvent
        fields = {}



import phonenumbers
from celery import shared_task
from django.contrib.postgres.fields import ArrayField
from django.core.exceptions import MultipleObjectsReturned
from django.db import models, transaction
from django.db.models import Case, CharField, F, Q, When
from django.db.models.signals import m2m_changed
from django.dispatch import receiver
from wbcore.contrib.directory.models import EmailContact, Person, TelephoneContact
from wbcore.contrib.notifications.utils import create_notification_type
from wbcore.models import WBModel

from wbintegrator_office365.importer import MicrosoftGraphAPI

from .tenant import TenantUser


class Event(WBModel):
    class Type(models.TextChoices):
        CALLRECORD = "CALLRECORD", "Call Record"
        CALENDAR = "CALENDAR", "Calendar"

    class Meta:
        verbose_name = "Event"
        verbose_name_plural = "Events"
        permissions = [("administrate_event", "Can Administrate call and calendar events")]

    auto_inc_id = models.IntegerField(default=0)
    nb_received = models.IntegerField(default=0)

    uuid_event = models.CharField(
        max_length=1000,
        verbose_name="Event UID",
        help_text="UUID obtained from Microsoft",
        null=True,
        blank=True,
        unique=True,
    )
    id_event = models.CharField(
        max_length=1000,
        verbose_name="Event ID",
        help_text="Event ID obtained from Microsoft",
        null=True,
        blank=True,
        unique=True,
    )
    type = models.CharField(
        max_length=32, choices=Type.choices, null=True, blank=True, verbose_name="Type", default=""
    )
    subscription_id = models.CharField(
        max_length=255, null=True, blank=True, verbose_name="Subscription ID", default=""
    )
    change_type = models.CharField(max_length=255, null=True, blank=True, verbose_name="Change Type", default="")
    resource = models.CharField(max_length=1000, null=True, blank=True, verbose_name="Resource", default="")
    created = models.DateTimeField(auto_now_add=True, verbose_name="Created")
    changed = models.DateTimeField(auto_now=True, verbose_name="Changed")
    is_handled = models.BooleanField(default=True)
    data = models.JSONField(default=dict, null=True, blank=True)

    tenant_user = models.ForeignKey(
        "wbintegrator_office365.TenantUser",
        related_name="tenant_events",
        null=True,
        blank=True,
        on_delete=models.deletion.SET_NULL,
        verbose_name="Tenant User",
    )

    def save(self, *args, **kwargs):
        events = Event.objects.filter(id=self.id)
        object_list = Event.objects.order_by("auto_inc_id")
        if len(events) == 0:
            self.nb_received = 1
            if len(object_list) == 0:  # if there are no objects
                self.auto_inc_id = 1
            else:
                self.auto_inc_id = object_list.last().auto_inc_id + 1
        else:
            self.nb_received = events.first().nb_received + 1
            self.auto_inc_id = events.first().auto_inc_id
        super().save(*args, **kwargs)

    def __str__(self):
        return f"{self.id}"

    @classmethod
    def is_administrator(cls, user):
        user_groups = user.groups
        user_permission = user.user_permissions
        return (
            user_groups.filter(permissions__codename="administrate_event").exists()
            or user_permission.filter(codename="administrate_event").exists()
            or user.is_superuser
        )

    @classmethod
    def get_endpoint_basename(self):
        return "wbintegrator_office365:event"

    @classmethod
    def get_representation_endpoint(cls):
        return "wbintegrator_office365:eventrepresentation-list"

    @classmethod
    def get_representation_value_key(cls):
        return "id"

    @classmethod
    def get_representation_label_key(cls):
        return "{{id_str}} - {{uuid_event}}"

    def fetch_call(self):
        if data := MicrosoftGraphAPI().call(self.uuid_event):
            # call user organiser
            is_user = False
            is_guest = False
            is_phone = False
            organizer_id = None
            organizer_display_name = None
            organizer_organization_id = None
            if data.get("organizer.user.id") or (
                data.get("organizer.user.id") == "" and data.get("organizer.user.tenantId")
            ):
                is_user = True
                organizer_id = data.get("organizer.user.id")
                organizer_display_name = data.get("organizer.user.display_name")
                organizer_organization_id = data.get("organizer.user.tenantId")
            elif data.get("organizer.guest.id") or (
                data.get("organizer.guest.id") == "" and data.get("organizer.guest.tenantId")
            ):
                is_guest = True
                organizer_id = data.get("organizer.guest.id")
                organizer_display_name = data.get("organizer.guest.display_name")
                organizer_organization_id = data.get("organizer.guest.tenantId")
            elif data.get("organizer.phone.id") or (
                data.get("organizer.phone.id") == "" and data.get("organizer.phone.tenantId")
            ):
                is_phone = True
                organizer_id = data.get("organizer.phone.id")
                organizer_display_name = data.get("organizer.phone.display_name")
                organizer_organization_id = data.get("organizer.phone.tenantId")

            call_user_organizer = create_or_update_call_user(
                tenant_id=organizer_id,
                display_name=organizer_display_name,
                tenant_organization_id=organizer_organization_id,
                is_user=is_user,
                is_guest=is_guest,
                is_phone=is_phone,
                acs_user=data.get("organizer.acs_user"),
                splool_user=data.get("organizer.splool_user"),
                encrypted=data.get("organizer.encrypted"),
                on_premises=data.get("organizer.on_premises"),
                acs_application_instance=data.get("organizer.acs_application_instance"),
                spool_application_instance=data.get("organizer.spool_application_instance"),
                application_instance=data.get("organizer.application_instance"),
                application=data.get("organizer.application"),
                device=data.get("organizer.device"),
            )

            # CALL EVENT
            try:
                call, created = CallEvent.objects.update_or_create(
                    event=self,
                    defaults={
                        "version": data.get("version"),
                        "type": data.get("type"),
                        "modalities": data.get("modalities"),
                        "last_modified": data.get("last_modified"),
                        "start": data.get("start"),
                        "end": data.get("end"),
                        "join_web_url": data.get("join_web_url"),
                        "organizer": call_user_organizer,
                        "data": data,
                    },
                )
            except MultipleObjectsReturned:
                call = CallEvent.objects.filter(event=self).order_by("-pk")[0]
                CallEvent.objects.filter(event=self).exclude(id=call.id).delete()
                CallEvent.objects.filter(event=self).update(
                    version=data.get("version"),
                    type=data.get("type"),
                    modalities=data.get("modalities"),
                    last_modified=data.get("last_modified"),
                    start=data.get("start"),
                    end=data.get("end"),
                    join_web_url=data.get("join_web_url"),
                    organizer=call_user_organizer,
                    data=data,
                )

            # call user for all participants
            for participant in data.get("participants"):
                is_user = False
                is_guest = False
                is_phone = False
                participant_id = None
                participant_display_name = None
                participant_organization_id = None
                if participant.get("user"):
                    is_user = True
                    participant_id = participant.get("user").get("id")
                    participant_display_name = participant.get("user").get("displayName")
                    participant_organization_id = participant.get("user").get("tenantId")
                elif participant.get("guest"):
                    is_guest = True
                    participant_id = participant.get("guest").get("id")
                    participant_display_name = participant.get("guest").get("displayName")
                    participant_organization_id = participant.get("guest").get("tenantId")
                elif participant.get("phone"):
                    is_phone = True
                    participant_id = participant.get("phone").get("id")
                    participant_display_name = participant.get("phone").get("displayName")
                    participant_organization_id = participant.get("phone").get("tenantId")

                call_user = create_or_update_call_user(
                    tenant_id=participant_id,
                    display_name=participant_display_name,
                    tenant_organization_id=participant_organization_id,
                    is_user=is_user,
                    is_guest=is_guest,
                    is_phone=is_phone,
                    acs_user=participant.get("acsUser"),
                    splool_user=participant.get("spoolUser"),
                    encrypted=participant.get("encrypted"),
                    on_premises=participant.get("on_premises"),
                    acs_application_instance=participant.get("acsApplicationInstance"),
                    spool_application_instance=participant.get("spoolApplicationInstance"),
                    application_instance=participant.get("applicationInstance"),
                    application=participant.get("application"),
                    device=participant.get("device"),
                )
                call.participants.add(call_user)


class EventLog(WBModel):
    class Meta:
        verbose_name = "Event Log"
        verbose_name_plural = "Event Logs"

    last_event = models.ForeignKey(
        "Event",
        related_name="event_logs",
        verbose_name="Last Event",
        on_delete=models.deletion.CASCADE,
    )
    id_event = models.CharField(max_length=1000, null=True, blank=True, verbose_name="Event ID")
    order_received = models.IntegerField(default=0)
    change_type = models.CharField(max_length=255, null=True, blank=True, verbose_name="Change Type", default="")
    created = models.DateTimeField(auto_now_add=True, verbose_name="Created")
    changed = models.DateTimeField(auto_now=True, verbose_name="Changed")
    resource = models.CharField(max_length=1000, null=True, blank=True, verbose_name="Resource", default="")
    is_handled = models.BooleanField(default=True)
    data = models.JSONField(default=dict, null=True, blank=True)

    @classmethod
    def get_endpoint_basename(self):
        return "wbintegrator_office365:eventlog"

    @classmethod
    def get_representation_endpoint(cls):
        return "wbintegrator_office365:eventlogrepresentation-list"

    @classmethod
    def get_representation_value_key(cls):
        return "id"

    @classmethod
    def get_representation_label_key(cls):
        return "{{last_event}} ({{order_received}})"


class CallUser(WBModel):
    acs_user = models.CharField(max_length=256, default="", null=True, blank=True)
    splool_user = models.CharField(max_length=256, default="", null=True, blank=True)
    encrypted = models.CharField(max_length=256, default="", null=True, blank=True)
    on_premises = models.CharField(max_length=256, default="", null=True, blank=True)
    acs_application_instance = models.CharField(max_length=256, default="", null=True, blank=True)
    spool_application_instance = models.CharField(max_length=256, default="", null=True, blank=True)
    application_instance = models.CharField(max_length=256, default="", null=True, blank=True)
    application = models.CharField(max_length=256, default="", null=True, blank=True)
    device = models.CharField(max_length=256, default="", null=True, blank=True)
    is_guest = models.BooleanField(default=False)
    is_phone = models.BooleanField(default=False)
    tenant_user = models.ForeignKey(
        "wbintegrator_office365.TenantUser",
        related_name="call_users",
        null=True,
        blank=True,
        on_delete=models.deletion.SET_NULL,
    )

    def __str__(self):
        if self.tenant_user:
            if self.tenant_user.profile:
                return f"{self.tenant_user.profile.computed_str}"
            elif self.tenant_user.mail or self.tenant_user.display_name:
                mail = self.tenant_user.mail if self.tenant_user.mail else self.tenant_user.id
                return f"{self.tenant_user.display_name}({mail})"
            else:
                return f"{self.tenant_user.tenant_id}"
        return f"{self.id}"

    @classmethod
    def annotated_queryset(cls, qs):
        return qs.annotate(
            tenant_user_str=Case(
                When(tenant_user__isnull=True, then=F("id_str")),
                default=F("tenant_user__profile_str"),
                output_field=CharField(),
            ),
        )

    @classmethod
    def get_endpoint_basename(cls):
        return "wbintegrator_office365:calluser"

    @classmethod
    def get_representation_endpoint(cls):
        return "wbintegrator_office365:calluserrepresentation-list"

    @classmethod
    def get_representation_value_key(cls):
        return "id"

    @classmethod
    def get_representation_label_key(cls):
        return "{{tenant_user_str}}"


class CallEvent(WBModel):
    class Meta:
        verbose_name = "Call Event"
        verbose_name_plural = "Calls Events"

        notification_types = [
            create_notification_type(
                "wbintegrator_office365.callevent.notify",
                "Call Summary",
                "Call Summary",
                False,
                False,
                True,
                is_lock=True,
            ),
            create_notification_type(
                "wbintegrator_office365.callevent.notify",
                "Call Event Notification",
                "Sends a notification when something happens with a Call Event triggered from Office 365",
                True,
                True,
                False,
            ),
        ]

    event = models.OneToOneField(to="wbintegrator_office365.Event", related_name="calls", on_delete=models.CASCADE)
    version = models.CharField(max_length=256, null=True, blank=True, verbose_name="Version", default="")
    type = models.CharField(max_length=256, null=True, blank=True, verbose_name="Type", default="")
    modalities = ArrayField(
        models.CharField(max_length=256), blank=True, null=True, verbose_name="Modalities", default=list
    )
    last_modified = models.DateTimeField(null=True, blank=True, verbose_name="Last Modified")
    start = models.DateTimeField(null=True, blank=True, verbose_name="Start")
    end = models.DateTimeField(null=True, blank=True, verbose_name="End")
    join_web_url = models.CharField(max_length=256, null=True, blank=True, verbose_name="Join Web Url", default="")
    organizer = models.ForeignKey(
        CallUser, verbose_name="Organizer", null=True, blank=True, on_delete=models.deletion.SET_NULL
    )
    participants = models.ManyToManyField(
        CallUser, blank=True, related_name="participates", verbose_name="Call participants"
    )
    activity = models.OneToOneField(
        "wbcrm.Activity",
        related_name="call_event",
        null=True,
        blank=True,
        on_delete=models.deletion.SET_NULL,
        verbose_name="Activity",
    )
    created = models.DateTimeField(auto_now_add=True, verbose_name="Created")
    is_internal_call = models.BooleanField(null=True, blank=True, verbose_name="Is internal call")
    data = models.JSONField(default=dict, null=True, blank=True)

    def __str__(self):
        return f"{self.event.id} ({self.start} - {self.end})"

    def check_call_is_internal_call(self, caller_id=None):
        is_internal = True
        callers = self.participants.all()

        if caller_id:
            is_internal = self.is_internal_call if self.is_internal_call is not None else is_internal
            callers = callers.filter(id=caller_id)

        for caller in callers:
            if (tenant_user := caller.tenant_user) and (person := tenant_user.profile) and person.is_internal:
                is_internal &= True
            else:
                is_internal &= False

        self.is_internal_call = is_internal
        self.save()

    @classmethod
    def get_endpoint_basename(cls):
        return "wbintegrator_office365:callevent"

    @classmethod
    def get_representation_endpoint(cls):
        return "wbintegrator_office365:calleventrepresentation-list"

    @classmethod
    def get_representation_value_key(cls):
        return "id"

    @classmethod
    def get_representation_label_key(cls):
        return "{{event.id}} ({{start}} - {{end}})"


# search person by phone if not found
def get_person_by_phone(phone):
    persons = Person.objects.none()
    phone_numbers = None
    try:
        parser_number = phonenumbers.parse(phone, "CH")
        phone_numbers = (
            phonenumbers.format_number(parser_number, phonenumbers.PhoneNumberFormat.E164)
            if parser_number
            else "".join(phone.split())
        )
        if phone and phone_numbers:
            phone2 = "".join(phone.replace("+41", "").split())
            entries = (
                TelephoneContact.objects.filter(
                    Q(number__contains=phone) | Q(number__contains=phone2) | Q(number__contains=phone_numbers)
                )
                .values("entry")
                .distinct()
            )
            persons = Person.objects.filter(id__in=entries)
    except Exception as e:
        print(e, phone)  # noqa: T201
    return persons, phone_numbers


def fetch_tenantusers():
    datum = MicrosoftGraphAPI().users()
    count_added = 0
    if datum:
        for data in datum:
            # username = user.get("email")
            # if user.get("display_name") and user.get("id"):
            #     username = re.sub(' +', ' ', user.get("display_name").lower()).replace(",", "").replace(" ", "-").strip()
            tenant_id = data.get("id")
            display_name = data.get("display_name")
            mail = data.get("mail") if data.get("mail") else data.get("user_principal_name")
            phone = (
                next(iter(data.get("business_phones") or []), None)
                if data.get("business_phones")
                else data.get("mobile_phone")
            )
            # print(display_name,"/", mail,"/", phone,"/", data.get("given_name"),"/", data.get("surname"))
            # 1) search person by email
            entries = EmailContact.objects.filter(address=mail).values("entry").distinct()
            persons = Person.objects.filter(id__in=entries)
            phone_numbers = None
            # 2) search person by phone if not found
            if persons.count() == 0 and phone:
                persons, phone_numbers = get_person_by_phone(phone)
            # 3) search person by surname and given_name if not found
            if persons.count() == 0 and data.get("surname") and data.get("given_name"):
                persons = Person.objects.filter(
                    first_name__icontains=data.get("given_name"), last_name__icontains=data.get("surname")
                ).distinct()
            # get the first person found otherwise it will be None
            person = None
            if persons.count() > 0:
                person = persons.first()
            else:
                first_name, last_name = (
                    display_name.split(" ") if len(display_name.split(" ")) == 2 else ["AnonymousUser", tenant_id]
                )
                qs_person = Person.objects.filter(first_name=first_name, last_name=last_name)
                if qs_person.count() > 0:
                    person = qs_person.first()
                # person, created = Person.objects.get_or_create(first_name=first_name, last_name=last_name)
                # if mail:
                #     EmailContact.objects.get_or_create(
                #         primary=True,
                #         entry=person,
                #         address=mail,
                #     )
                # if phone_numbers:
                #     TelephoneContact.objects.get_or_create(
                #         entry=person,
                #         number=phone_numbers
                #     )

            # update or create tenant user
            tenantuser, created = TenantUser.objects.update_or_create(
                tenant_id=tenant_id,
                defaults={"display_name": display_name, "mail": mail, "phone": phone, "profile": person},
            )
            if created:
                count_added += 1
    return datum, count_added


def create_or_update_call_user(
    tenant_id=None,
    display_name=None,
    tenant_organization_id=None,
    is_user=False,
    is_guest=False,
    is_phone=False,
    acs_user=None,
    splool_user=None,
    encrypted=None,
    on_premises=None,
    acs_application_instance=None,
    spool_application_instance=None,
    application_instance=None,
    application=None,
    device=None,
):
    tenant_user = None
    # get or create Person
    if tenant_id:
        # search person by tenant id
        qs_tenant_user = TenantUser.objects.filter(tenant_id=tenant_id)
        if qs_tenant_user.count() == 0:
            first_name, last_name = ["AnonymousUser", tenant_id]
            phone_numbers = None
            persons = Person.objects.none()
            person = None
            # search person by display name  (also for phone anonymous) # the phone number obtained is "anonymous"
            if is_user or is_guest or (is_phone and tenant_id == "anonymous"):
                # first_name, last_name = display_name.split(" ") if len(display_name.split(" ")) == 2 and display_name != "Guest User" and display_name != "External user" else first_name, last_name
                # Sometimes we obtained "Guest User" or "External user" for certains external user
                if display_name:
                    if (
                        len(display_name.strip(" ").split(" ")) == 2
                        and display_name.lower() != "guest user"
                        and display_name.lower() != "external user"
                    ):
                        first_name, last_name = display_name.strip(" ").split(" ")
                    else:
                        first_name = display_name

                persons = Person.objects.filter(
                    first_name__icontains=first_name, last_name__icontains=last_name
                ).distinct()
            # search person by phone
            elif is_phone and tenant_id != "anonymous":
                persons, phone_numbers = get_person_by_phone(tenant_id)

            if persons.count():
                person = persons.first()
            else:
                qs_person = Person.objects.filter(first_name=first_name, last_name=last_name)
                if qs_person.count() > 0:
                    person = qs_person.first()
                # person, created = Person.objects.get_or_create(first_name=first_name, last_name=last_name)
                # if phone_numbers:
                #     TelephoneContact.objects.get_or_create(
                #         entry=person,
                #         number=phone_numbers
                #     )
                #     phone_numbers = tenant_id

            tenant_user, created = TenantUser.objects.get_or_create(
                tenant_id=tenant_id, defaults={"phone": phone_numbers, "display_name": display_name, "profile": person}
            )
        else:
            tenant_user = TenantUser.objects.get(tenant_id=tenant_id)

        if tenant_organization_id:
            tenant_user.tenant_organization_id = tenant_organization_id
            tenant_user.save()
    else:
        tenant_user, _ = TenantUser.objects.get_or_create(
            tenant_id=tenant_id, display_name=display_name, tenant_organization_id=tenant_organization_id
        )

    call_user, created = CallUser.objects.update_or_create(
        acs_user=acs_user,
        splool_user=splool_user,
        encrypted=encrypted,
        on_premises=on_premises,
        acs_application_instance=acs_application_instance,
        spool_application_instance=spool_application_instance,
        application_instance=application_instance,
        application=application,
        device=device,
        tenant_user=tenant_user,
        is_phone=is_phone,
        is_guest=is_guest,
    )
    return call_user


@shared_task
def handle_event_from_webhook(id_event, notification):
    with transaction.atomic():
        from wbintegrator_office365.models.subscription import Subscription

        if (
            (subscription_id := notification.get("subscription_id"))
            and (_subscription := Subscription.objects.filter(subscription_id=subscription_id).first())
            and _subscription.type_resource == Subscription.TypeResource.CALLRECORD
        ):
            event, _ = Event.objects.update_or_create(
                uuid_event=id_event,
                defaults={
                    "type": _subscription.type_resource,
                    "subscription_id": subscription_id,
                    "change_type": notification.get("change_type"),
                    "data": notification,
                    "resource": notification.get("resource"),
                    "is_handled": True,
                    "tenant_user": _subscription.tenant_user,
                },
            )
            event.fetch_call()


@receiver(m2m_changed, sender=CallEvent.participants.through)
def post_save_participant_event(sender, instance, action, reverse, pk_set, *args, **kwargs):
    if action == "post_add" and not reverse:
        for participant_id in pk_set:
            instance.check_call_is_internal_call(participant_id)
    elif action == "post_remove" and not reverse:
        instance.check_call_is_internal_call()



# Generated by Django 4.1.7 on 2023-04-19 05:32

import django.contrib.postgres.fields
import django.core.validators
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("directory", "0001_initial"),
        ("wbcrm", "0001_initial_squashed_squashed_0032_productcompanyrelationship_alter_product_prospects_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="Address",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("type", models.CharField(blank=True, default="", max_length=255, null=True)),
                ("street", models.CharField(blank=True, default="", max_length=255, null=True)),
                ("city", models.CharField(blank=True, default="", max_length=255, null=True)),
                ("state", models.CharField(blank=True, default="", max_length=255, null=True)),
                (
                    "country_or_region",
                    models.CharField(blank=True, default="", max_length=255, null=True, verbose_name="Country/Region"),
                ),
                ("postal_code", models.CharField(blank=True, default="", max_length=255, null=True)),
            ],
            options={
                "verbose_name": "Address",
                "verbose_name_plural": "Addresses",
            },
        ),
        migrations.CreateModel(
            name="RecurrencePattern",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "type",
                    models.CharField(blank=True, default="", max_length=255, null=True, verbose_name="Pattern type"),
                ),
                ("interval", models.IntegerField(blank=True, default=0, null=True)),
                (
                    "month",
                    models.IntegerField(
                        blank=True,
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(1),
                            django.core.validators.MaxValueValidator(12),
                        ],
                    ),
                ),
                (
                    "day_of_month",
                    models.IntegerField(
                        blank=True,
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(1),
                            django.core.validators.MaxValueValidator(30),
                        ],
                    ),
                ),
                (
                    "days_of_week",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.CharField(default="", max_length=256), blank=True, null=True, size=None
                    ),
                ),
                ("first_day_of_week", models.CharField(blank=True, default="", max_length=255, null=True)),
                (
                    "index",
                    models.CharField(blank=True, default="", max_length=255, null=True, verbose_name="Pattern index"),
                ),
                ("range_type", models.CharField(blank=True, default="", max_length=255, null=True)),
                ("start_date", models.DateField(blank=True, null=True)),
                ("end_date", models.DateField(blank=True, null=True)),
                ("recurrence_time_zone", models.CharField(blank=True, default="", max_length=255, null=True)),
                ("number_of_occurrences", models.IntegerField(blank=True, default=0, null=True)),
            ],
            options={
                "verbose_name_plural": "Recurrence Patterns",
                "verbose_name": "Recurrence Pattern",
            },
        ),
        migrations.CreateModel(
            name="TenantUser",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "tenant_id",
                    models.CharField(blank=True, default="", max_length=255, null=True, verbose_name="User tenant ID"),
                ),
                (
                    "display_name",
                    models.CharField(blank=True, default="", max_length=255, null=True, verbose_name="Display Name"),
                ),
                ("mail", models.CharField(blank=True, default="", max_length=255, null=True)),
                ("phone", models.CharField(blank=True, default="", max_length=255, null=True)),
                (
                    "tenant_organization_id",
                    models.CharField(blank=True, max_length=255, null=True, verbose_name="Organization tenant ID"),
                ),
                ("enable_synchronization_activity", models.BooleanField(default=False)),
                ("is_internal_organization", models.BooleanField(blank=True, null=True)),
                (
                    "profile",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="tenant_user",
                        to="directory.person",
                        verbose_name="User",
                    ),
                ),
            ],
            options={
                "verbose_name": "Tenant User",
                "verbose_name_plural": "Tenant Users",
            },
        ),
        migrations.CreateModel(
            name="Location",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("unique_id", models.CharField(blank=True, default="", max_length=255, null=True)),
                ("unique_id_type", models.CharField(blank=True, default="", max_length=255, null=True)),
                (
                    "display_name",
                    models.CharField(blank=True, default="", max_length=255, null=True, verbose_name="Name"),
                ),
                (
                    "location_uri",
                    models.CharField(blank=True, default="", max_length=255, null=True, verbose_name="Reply URL"),
                ),
                ("type", models.CharField(blank=True, default="", max_length=255, null=True)),
                (
                    "address",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="wbintegrator_office365.address",
                        verbose_name="Address",
                    ),
                ),
            ],
            options={
                "verbose_name": "Location Meeting",
                "verbose_name_plural": "Location Meetings",
            },
        ),
        migrations.CreateModel(
            name="CallUser",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("acs_user", models.CharField(blank=True, default="", max_length=256, null=True)),
                ("splool_user", models.CharField(blank=True, default="", max_length=256, null=True)),
                ("encrypted", models.CharField(blank=True, default="", max_length=256, null=True)),
                ("on_premises", models.CharField(blank=True, default="", max_length=256, null=True)),
                ("acs_application_instance", models.CharField(blank=True, default="", max_length=256, null=True)),
                ("spool_application_instance", models.CharField(blank=True, default="", max_length=256, null=True)),
                ("application_instance", models.CharField(blank=True, default="", max_length=256, null=True)),
                ("application", models.CharField(blank=True, default="", max_length=256, null=True)),
                ("device", models.CharField(blank=True, default="", max_length=256, null=True)),
                ("is_guest", models.BooleanField(default=False)),
                ("is_phone", models.BooleanField(default=False)),
                (
                    "tenant_user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="call_users",
                        to="wbintegrator_office365.tenantuser",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Calendar",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("created", models.DateTimeField(auto_now_add=True, verbose_name="Created")),
                ("last_modified", models.DateTimeField(auto_now=True, verbose_name="Last modified")),
                ("is_removable", models.BooleanField(default=False)),
                ("is_default_calendar", models.BooleanField(default=False)),
                (
                    "allowed_online_meeting_providers",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.CharField(max_length=256), blank=True, null=True, size=None
                    ),
                ),
                ("calendar_id", models.CharField(max_length=1000, unique=True, verbose_name="Calendar Id")),
                ("can_share", models.BooleanField(default=True)),
                ("can_edit", models.BooleanField(default=True)),
                ("can_view_private_items", models.BooleanField(default=True)),
                ("change_key", models.CharField(blank=True, max_length=255, null=True)),
                ("color", models.CharField(blank=True, max_length=255, null=True)),
                ("default_online_meeting_provider", models.CharField(blank=True, max_length=255, null=True)),
                ("hex_color", models.CharField(blank=True, max_length=255, null=True)),
                ("is_tallying_responses", models.BooleanField(default=True)),
                ("name", models.CharField(blank=True, max_length=255, null=True)),
                (
                    "odata_context",
                    models.CharField(blank=True, max_length=255, null=True, verbose_name="Odata Context"),
                ),
                ("data", models.JSONField(blank=True, default=dict, null=True)),
                ("is_shared", models.BooleanField(default=True)),
                ("is_shared_with_me", models.BooleanField(default=False)),
                ("is_enable", models.BooleanField(default=True)),
                ("outlook_verification", models.BooleanField(default=False)),
                (
                    "owner",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="office_calendars",
                        to="directory.person",
                        verbose_name="Owner",
                    ),
                ),
            ],
            options={
                "verbose_name": "Calendar",
                "verbose_name_plural": "Calendars",
            },
        ),
        migrations.CreateModel(
            name="Subscription",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("is_enable", models.BooleanField(default=True)),
                ("subscription_id", models.CharField(blank=True, max_length=255, null=True)),
                ("change_type", models.CharField(blank=True, max_length=255, null=True)),
                ("notification_url", models.CharField(blank=True, max_length=255, null=True)),
                ("resource", models.CharField(blank=True, max_length=255, null=True)),
                ("expiration_date", models.DateTimeField(blank=True, null=True, verbose_name="Expiration Date")),
                ("application_id", models.CharField(blank=True, max_length=255, null=True)),
                ("creator_id", models.CharField(blank=True, max_length=255, null=True)),
                ("client_state", models.CharField(blank=True, max_length=255, null=True)),
                ("latest_supported_tls_version", models.CharField(blank=True, max_length=255, null=True)),
                ("notification_content_type", models.CharField(blank=True, max_length=255, null=True)),
                ("odata_context", models.CharField(blank=True, max_length=255, null=True)),
                ("encryption_certificate_id", models.CharField(blank=True, max_length=255, null=True)),
                ("encryption_certificate", models.CharField(blank=True, max_length=255, null=True)),
                ("include_resource_data", models.CharField(blank=True, max_length=255, null=True)),
                ("notification_query_options", models.CharField(blank=True, max_length=255, null=True)),
                (
                    "type_resource",
                    models.CharField(
                        choices=[("CALLRECORD", "Call Record"), ("CALENDAR", "Calendar")],
                        default="CALLRECORD",
                        max_length=32,
                        verbose_name="Type of the resource",
                    ),
                ),
                (
                    "tenant_user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="subscriptions",
                        to="wbintegrator_office365.tenantuser",
                        verbose_name="Tenant User",
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True, verbose_name="Created")),
            ],
            options={
                "verbose_name": "Subscription",
                "verbose_name_plural": "Subscriptions",
            },
        ),
        migrations.CreateModel(
            name="Event",
            fields=[
                ("auto_inc_id", models.IntegerField(default=0)),
                ("nb_received", models.IntegerField(default=0)),
                (
                    "uuid_event",
                    models.CharField(
                        blank=True,
                        help_text="UUID obtained from Microsoft",
                        max_length=1000,
                        null=True,
                        unique=True,
                        verbose_name="Event UID",
                    ),
                ),
                (
                    "type",
                    models.CharField(
                        blank=True,
                        choices=[("CALLRECORD", "Call Record"), ("CALENDAR", "Calendar")],
                        default="",
                        max_length=32,
                        null=True,
                        verbose_name="Type",
                    ),
                ),
                (
                    "subscription_id",
                    models.CharField(
                        blank=True, default="", max_length=255, null=True, verbose_name="Subscription ID"
                    ),
                ),
                (
                    "change_type",
                    models.CharField(blank=True, default="", max_length=255, null=True, verbose_name="Change Type"),
                ),
                (
                    "resource",
                    models.CharField(blank=True, default="", max_length=1000, null=True, verbose_name="Resource"),
                ),
                ("created", models.DateTimeField(auto_now_add=True, verbose_name="Created")),
                ("data", models.JSONField(blank=True, default=dict, null=True)),
                ("changed", models.DateTimeField(auto_now=True, verbose_name="Changed")),
                (
                    "tenant_user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="tenant_events",
                        to="wbintegrator_office365.tenantuser",
                        verbose_name="Tenant User",
                    ),
                ),
                (
                    "id_event",
                    models.CharField(
                        blank=True,
                        help_text="Event ID obtained from Microsoft",
                        max_length=1000,
                        null=True,
                        unique=True,
                        verbose_name="Event ID",
                    ),
                ),
                ("is_handled", models.BooleanField(default=True)),
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
            ],
            options={
                "verbose_name": "Event",
                "verbose_name_plural": "Events",
                "permissions": [("admin_event", "Can administer call and calendar events")],
            },
        ),
        migrations.CreateModel(
            name="CalendarEvent",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "odata_context",
                    models.CharField(blank=True, max_length=255, null=True, verbose_name="Odata Context"),
                ),
                ("odata_etag", models.CharField(blank=True, max_length=255, null=True, verbose_name="Odata Context")),
                (
                    "created_calendar",
                    models.DateTimeField(blank=True, null=True, verbose_name="Created in the Calendar"),
                ),
                (
                    "last_modified_calendar",
                    models.DateTimeField(blank=True, null=True, verbose_name="Last modified in the Calendar"),
                ),
                ("created", models.DateTimeField(auto_now_add=True, verbose_name="Created")),
                ("last_modified", models.DateTimeField(auto_now=True, verbose_name="Last modified")),
                (
                    "categories",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.CharField(max_length=256), blank=True, null=True, size=None
                    ),
                ),
                ("change_key", models.CharField(blank=True, max_length=255, null=True)),
                ("transaction_id", models.CharField(blank=True, max_length=255, null=True)),
                ("original_start_time_zone", models.CharField(blank=True, max_length=255, null=True)),
                ("original_end_time_zone", models.CharField(blank=True, max_length=255, null=True)),
                ("reminder_minutes_before_start", models.IntegerField(default=15)),
                ("is_reminder_on", models.BooleanField(default=False)),
                ("has_attachments", models.BooleanField(default=False)),
                ("subject", models.CharField(default="(No subject)", max_length=255)),
                ("body_preview", models.TextField(blank=True, default="", null=True)),
                ("importance", models.CharField(blank=True, max_length=255, null=True)),
                ("sensitivity", models.CharField(blank=True, max_length=255, null=True)),
                ("is_all_day", models.BooleanField(default=False)),
                ("is_cancelled", models.BooleanField(default=False)),
                ("is_organizer", models.BooleanField(blank=True, null=True)),
                ("response_requested", models.BooleanField(blank=True, null=True)),
                ("series_master_id", models.CharField(blank=True, max_length=255, null=True)),
                ("show_as", models.CharField(blank=True, max_length=255, null=True)),
                ("type", models.CharField(blank=True, max_length=255, null=True)),
                ("web_link", models.CharField(blank=True, max_length=255, null=True)),
                ("online_meeting_url", models.CharField(blank=True, max_length=255, null=True)),
                ("is_online_meeting", models.BooleanField(blank=True, null=True)),
                ("online_meeting_provider", models.CharField(blank=True, max_length=255, null=True)),
                ("allow_new_time_proposals", models.BooleanField(blank=True, null=True)),
                ("occurrence_id", models.CharField(blank=True, max_length=255, null=True)),
                ("is_draft", models.BooleanField(blank=True, null=True)),
                ("hide_attendees", models.BooleanField(blank=True, null=True)),
                ("response_status", models.CharField(blank=True, max_length=255, null=True)),
                ("body", models.TextField(blank=True, default="", null=True)),
                ("start", models.DateTimeField(blank=True, null=True, verbose_name="Start")),
                ("end", models.DateTimeField(blank=True, null=True, verbose_name="End")),
                (
                    "locations",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.CharField(default="", max_length=256), blank=True, null=True, size=None
                    ),
                ),
                ("on_line_meeting", models.CharField(blank=True, max_length=255, null=True)),
                ("data", models.JSONField(blank=True, default=dict, null=True)),
                (
                    "activity",
                    models.OneToOneField(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="calendar",
                        to="wbcrm.activity",
                        verbose_name="Activity",
                    ),
                ),
                (
                    "calendar",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="calendars",
                        to="wbintegrator_office365.calendar",
                        verbose_name="Calendar",
                    ),
                ),
                (
                    "recurrence",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="wbintegrator_office365.recurrencepattern",
                        verbose_name="Recurrence Pattern",
                    ),
                ),
                ("is_enable", models.BooleanField(default=True)),
                ("location", models.CharField(blank=True, max_length=255, null=True)),
                ("outlook_verification", models.BooleanField(default=False)),
                (
                    "invitation_event_ids",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.CharField(max_length=1000),
                        blank=True,
                        default=list,
                        help_text="list of invitation IDs received by participants",
                        size=None,
                        verbose_name="Invitation Event Ids",
                    ),
                ),
                (
                    "event",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="calendars_events",
                        to="wbintegrator_office365.event",
                        verbose_name="Event",
                    ),
                ),
                (
                    "organizer",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="organizer_office_calendar_events",
                        to="directory.person",
                        verbose_name="Organizer",
                    ),
                ),
                (
                    "participants",
                    models.ManyToManyField(
                        blank=True,
                        related_name="calendar_events_attendees",
                        to="directory.person",
                        verbose_name="Participants",
                    ),
                ),
            ],
            options={
                "verbose_name": "Calendar Event",
                "verbose_name_plural": "Calendar Events",
            },
        ),
        migrations.CreateModel(
            name="CallEvent",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "version",
                    models.CharField(blank=True, default="", max_length=256, null=True, verbose_name="Version"),
                ),
                ("type", models.CharField(blank=True, default="", max_length=256, null=True, verbose_name="Type")),
                (
                    "modalities",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.CharField(max_length=256),
                        blank=True,
                        default=list,
                        null=True,
                        size=None,
                        verbose_name="Modalities",
                    ),
                ),
                ("last_modified", models.DateTimeField(blank=True, null=True, verbose_name="Last Modified")),
                ("start", models.DateTimeField(blank=True, null=True, verbose_name="Start")),
                ("end", models.DateTimeField(blank=True, null=True, verbose_name="End")),
                (
                    "join_web_url",
                    models.CharField(blank=True, default="", max_length=256, null=True, verbose_name="Join Web Url"),
                ),
                ("created", models.DateTimeField(auto_now_add=True, verbose_name="Created")),
                (
                    "activity",
                    models.OneToOneField(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="call_event",
                        to="wbcrm.activity",
                        verbose_name="Activity",
                    ),
                ),
                (
                    "organizer",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="wbintegrator_office365.calluser",
                        verbose_name="Organizer",
                    ),
                ),
                (
                    "participants",
                    models.ManyToManyField(
                        blank=True,
                        related_name="participates",
                        to="wbintegrator_office365.calluser",
                        verbose_name="Call participants",
                    ),
                ),
                ("is_internal_call", models.BooleanField(blank=True, null=True, verbose_name="Is internal call")),
                ("data", models.JSONField(blank=True, default=dict, null=True)),
                (
                    "event",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="calls",
                        to="wbintegrator_office365.event",
                    ),
                ),
            ],
            options={
                "verbose_name": "Call Event",
                "verbose_name_plural": "Calls Events",
            },
        ),
        migrations.CreateModel(
            name="EventLog",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("order_received", models.IntegerField(default=0)),
                (
                    "change_type",
                    models.CharField(blank=True, default="", max_length=255, null=True, verbose_name="Change Type"),
                ),
                ("created", models.DateTimeField(auto_now_add=True, verbose_name="Created")),
                ("changed", models.DateTimeField(auto_now=True, verbose_name="Changed")),
                ("data", models.JSONField(blank=True, default=dict, null=True)),
                ("id_event", models.CharField(blank=True, max_length=1000, null=True, verbose_name="Event ID")),
                (
                    "resource",
                    models.CharField(blank=True, default="", max_length=1000, null=True, verbose_name="Resource"),
                ),
                ("is_handled", models.BooleanField(default=True)),
                (
                    "last_event",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="event_logs",
                        to="wbintegrator_office365.event",
                        verbose_name="Last Event",
                    ),
                ),
            ],
            options={
                "verbose_name": "Event Log",
                "verbose_name_plural": "Event Logs",
            },
        ),
        migrations.CreateModel(
            name="CalendarEventInstance",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("instance_id", models.CharField(max_length=1000, unique=True, verbose_name="Calendar Event Id")),
                ("start", models.DateTimeField(blank=True, null=True, verbose_name="Start")),
                ("end", models.DateTimeField(blank=True, null=True, verbose_name="End")),
                ("subject", models.CharField(default="(No subject)", max_length=255)),
                ("body", models.TextField(blank=True, default="", null=True)),
                ("body_preview", models.TextField(blank=True, default="", null=True)),
                ("created", models.DateTimeField(auto_now_add=True, verbose_name="Created")),
                ("last_modified", models.DateTimeField(auto_now=True, verbose_name="Last modified")),
                (
                    "activity",
                    models.OneToOneField(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="calendar_event_instance",
                        to="wbcrm.activity",
                        verbose_name="Activity",
                    ),
                ),
                (
                    "calendar_event",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="instances",
                        to="wbintegrator_office365.calendarevent",
                        verbose_name="Calendar Event",
                    ),
                ),
                (
                    "created_calendar",
                    models.DateTimeField(blank=True, null=True, verbose_name="Created in the Calendar"),
                ),
                ("is_enable", models.BooleanField(default=True)),
                (
                    "last_modified_calendar",
                    models.DateTimeField(blank=True, null=True, verbose_name="Last modified in the Calendar"),
                ),
                (
                    "organizer",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="organizer_office_calendar_event_instances",
                        to="directory.person",
                        verbose_name="Organizer",
                    ),
                ),
                (
                    "participants",
                    models.ManyToManyField(
                        blank=True,
                        related_name="calendar_events_instance_attendees",
                        to="directory.person",
                        verbose_name="Participants",
                    ),
                ),
            ],
            options={
                "verbose_name": "Calendar Event Instance",
                "verbose_name_plural": "Calendar Event Instances",
            },
        ),
    ]
